from __future__ import absolute_import, division, print_function, unicode_literals
import six

import copy
import unittest
import mock
import operator

from wad.detection import Detector, TIMEOUT
from wad.tests.data.data_test_wad import cern_ch_test_data


class TestDetector(unittest.TestCase):
    def setUp(self):
        self.detector = Detector()
        self.apps = self.detector.apps
        self.categories = self.detector.categories

    def test_check_re(self):
        # checking version patterns:
        #
        #   "headers": { "Server": "IIS(?:/([\\d.]+))?\\;version:\\1" },
        assert (self.detector.check_re(
            self.apps['IIS']['headers_re']['Server'],
            self.apps['IIS']['headers']['Server'],
            'Microsoft-IIS/7.5',
            [], None, 'IIS') == [{'app': 'IIS', 'ver': '7.5'}])

        # (?:maps\\.google\\.com/maps\\?file=api(?:&v=([\\d.]+))?|
        # maps\\.google\\.com/maps/api/staticmap)\\;version:API v\\1
        assert (self.detector.check_re(
            self.apps['Google Maps']['script_re'][0],
            self.apps['Google Maps']['script'][0],
            'abc <script src="maps.google.com/maps?file=api&v=123"> def',
            [], None, 'Google Maps') == [{'app': 'Google Maps', 'ver': 'API v123'}])

        # "script": [ "js/mage", "skin/frontend/(?:default|(enterprise))\\;version:\\1?Enterprise:Community" ],
        assert (self.detector.check_re(
            self.apps['Magento']['script_re'][1],
            self.apps['Magento']['script'][1],
            'abc <script src="skin/frontend/whatever"> def',
            [], None, 'Magento') == [])

        assert (self.detector.check_re(
            self.apps['Magento']['script_re'][1],
            self.apps['Magento']['script'][1],
            'abc <script src="skin/frontend/default"> def',
            [], None, 'Magento') == [{'app': 'Magento', 'ver': 'Community'}])

        assert (self.detector.check_re(
            self.apps['Magento']['script_re'][1],
            self.apps['Magento']['script'][1],
            'abc <script src="skin/frontend/enterprise"> def',
            [], None, 'Magento') == [{'app': 'Magento', 'ver': 'Enterprise'}])

    def test_check_url(self):
        assert self.detector.check_url("http://whatever.blogspot.com") == [{'app': 'Blogger', 'ver': None}]
        assert self.detector.check_url("https://whatever-else3414.de/script.php") == [{'app': 'PHP', 'ver': None}]

    def test_check_html(self):
        content = '<html><div id="gsNavBar" class="gcBorder1">whatever'
        assert self.detector.check_html(content) == [{'app': 'Gallery', 'ver': None}]

    def test_check_meta(self):
        assert (self.detector.check_meta('<html>    s<meta name="generator" content="Percussion">sssss    whatever') ==
                [{'app': 'Percussion', 'ver': None}])
        assert (self.detector.check_meta(" dcsaasd f<meta   name    = 'cargo_title' dd  content  =   'Pdafadfda'  >") ==
                [{'app': 'Cargo', 'ver': None}])
        assert (self.detector.check_meta(" dcsaasd f<mfffffffeta     name='cargo_title' dd  content='Pdafadfda'  >") ==
                [])
        assert self.detector.check_meta(" dcsaasd f<meta     name='cargo_title' >") == []

    def test_check_script(self):
        assert (self.detector.check_script('<html>    s<script  sda f     src    =  "jquery1.7.js">') ==
                [{'app': 'jQuery', 'ver': None}])
        assert self.detector.check_script(" dcsaasd f<script     src='' >") == []

    def test_check_headers(self):
        headers = [('Host', 'abc.com'), ('Server', 'Linux Ubuntu 12.10')]
        headers_mock = mock.Mock()
        headers_mock.items.return_value = headers

        assert (self.detector.check_headers(headers_mock) ==
                [{'app': 'Ubuntu', 'ver': None}])

    def test_implied_by(self):
        # ASP implies WS and IIS and IIS implies WS;
        # but we already know about IIS, so the only new implied app is WS
        assert self.detector.implied_by(['Microsoft ASP.NET', 'IIS']) == ['Windows Server']

    def test_follow_implies(self):
        # empty findings
        findings = []
        self.detector.follow_implies(findings)
        assert findings == []

        # no implies
        findings = [{'app': 'reCAPTCHA', 'ver': None}]
        self.detector.follow_implies(findings)
        assert findings == [{'app': 'reCAPTCHA', 'ver': None}]

        # Django CMS implies Django, and Django implies Python - let's see if this chain is followed
        findings = [{'app': 'Django CMS', 'ver': None}]
        self.detector.follow_implies(findings)
        assert (findings ==
                [{'app': 'Django CMS', 'ver': None},
                 {'app': 'Django', 'ver': None},
                 {'app': 'Python', 'ver': None}])

    def test_remove_duplicates(self):
        with_duplicates = [
            {'app': 'A', 'ver': None}, {'app': 'B', 'ver': "1.5"},
            {'app': 'C', 'ver': None}, {'app': 'D', 'ver': "7.0"},
            {'app': 'E', 'ver': "1"}, {'app': 'F', 'ver': "2.2"},
            {'app': 'A', 'ver': None}, {'app': 'B', 'ver': "1.5"},
            {'app': 'C', 'ver': "be"}, {'app': 'D', 'ver': "222"},
            {'app': 'A', 'ver': None}, {'app': 'B', 'ver': "1.5"},
            {'app': 'E', 'ver': None}, {'app': 'E', 'ver': "1.3"},
            {'app': 'F', 'ver': "2"}, {'app': 'F', 'ver': None},
        ]

        without_duplicates = [
            {'app': 'A', 'ver': None}, {'app': 'B', 'ver': "1.5"},
            {'app': 'C', 'ver': "be"}, {'app': 'D', 'ver': "7.0"},
            {'app': 'E', 'ver': "1.3"},
            {'app': 'F', 'ver': "2.2"}, {'app': 'D', 'ver': "222"},
        ]

        Detector().remove_duplicates(with_duplicates)
        assert with_duplicates == without_duplicates

    def test_excluded_by(self):
        # both 'TYPO3 Flow' and 'TYPO3 Neos' exclude 'TYPO3 CMS'
        assert self.detector.excluded_by(['TYPO3 Flow', 'TYPO3 Neos']) == ['TYPO3 CMS']
        # 'JBoss Web' excludes 'Apache Tomcat'; 'Mambo' excludes 'Joomla'
        assert set(self.detector.excluded_by(['Jo', 'JBoss Web', 'Jetty', 'Mambo'])) == set(['Joomla', 'Apache Tomcat'])
        # 'IIS' doesn't exclude anything
        assert self.detector.excluded_by(['IIS']) == []

    def test_remove_exclusions(self):
        # empty findings
        findings = []
        self.detector.remove_exclusions(findings)
        assert findings == []

        # no implies
        findings = [{'app': 'reCAPTCHA', 'ver': None}]
        self.detector.remove_exclusions(findings)
        assert findings == [{'app': 'reCAPTCHA', 'ver': None}]

        # real exclusions
        findings = [{'app': 'JBoss Web', 'ver': None},
                    {'app': 'Apache Tomcat', 'ver': None},
                    {'app': 'IIS', 'ver': None},
                    {'app': 'TYPO3 CMS', 'ver': None},
                    {'app': 'TYPO3 Flow', 'ver': None}]
        self.detector.remove_exclusions(findings)
        assert (findings ==
                [{'app': 'JBoss Web', 'ver': None},
                 {'app': 'IIS', 'ver': None},
                 {'app': 'TYPO3 Flow', 'ver': None}])

    def test_add_categories(self):
        findings = [
            {'app': 'Django CMS', 'ver': None},
            {'app': 'Django', 'ver': None},
            {'app': 'Python', 'ver': '2.7'},
            {'app': 'Dynamicweb', 'ver': 'beta'}]
        original = copy.deepcopy(findings)
        original[0]["type"] = "cms"
        original[1]["type"] = "web-frameworks"
        original[2]["type"] = "programming-languages"
        original[3]["type"] = "cms,ecommerce,analytics"

        self.detector.add_categories(findings)
        assert original == findings

    def test_url_match(self):
        assert self.detector.url_match(url='', regexp=None, default='test') == 'test'
        assert self.detector.url_match(url='example.com', regexp='exampl', default='test') is not None
        assert self.detector.url_match(url='example.com', regexp='ampl', default='test') is None

    def test_expected_url(self):
        url = "http://site.abc.com/dir/sub/script.php"
        assert self.detector.expected_url(url, None, None)
        assert self.detector.expected_url(url, 'http://.*abc.com/', None)
        assert not self.detector.expected_url(url, 'http://abc.com/', None)
        assert self.detector.expected_url(url, 'http://.*abc.com/', "php")
        assert not self.detector.expected_url(url, 'http://.*abc.com/', ".*php")
        assert self.detector.expected_url(url, None, ".*\.asp")
        assert not self.detector.expected_url(url, None, ".*\.php")

    def test_detect(self):
        expected = {
            'http://home.web.cern.ch/': [
                {'app': 'Apache',
                 'type': 'web-servers',
                 'ver': None},
                {'app': 'Drupal',
                 'type': 'cms',
                 'ver': '7'},
                {'app': 'Lightbox',
                 'type': 'photo-galleries,javascript-frameworks',
                 'ver': None},
                {'app': 'jQuery',
                 'type': 'javascript-frameworks',
                 'ver': None},
                {'app': 'Google Font API',
                 'type': 'font-scripts',
                 'ver': None},
                {'app': 'PHP',
                 'type': 'programming-languages',
                 'ver': None}
            ]
        }

        with mock.patch('wad.detection.tools') as mockObj:
            page = mock.Mock()
            page.geturl.return_value = cern_ch_test_data['geturl']
            if six.PY3:
                page.read.return_value = bytes(cern_ch_test_data['content'], encoding='utf-8')
            else:
                page.read.return_value = cern_ch_test_data['content']
            headers_mock = mock.Mock()
            headers_mock.items.return_value = cern_ch_test_data['headers'].items()
            page.info.return_value = headers_mock
            mockObj.urlopen = mock.Mock(return_value=page)
            results = self.detector.detect('http://cern.ch')
            assert list(six.iterkeys(results)) == list(six.iterkeys(expected))
            assert (sorted(next(six.itervalues(results)), key=operator.itemgetter('app')) ==
                    sorted(next(six.itervalues(expected)), key=operator.itemgetter('app')))

    def test_detect_multiple(self):
        urls_list = ["http://cern.ch", None, "", "http://cern.ch", "example.com"]
        with mock.patch('wad.detection.Detector.detect') as mockObj:
            mockObj.side_effect = [{'test1': 1}, {'test2': 2}]
            assert self.detector.detect_multiple(urls_list) == {'test1': 1, 'test2': 2}
            assert (('example.com', None, None, TIMEOUT),) in mockObj.call_args_list
            assert (('http://cern.ch', None, None, TIMEOUT),) in mockObj.call_args_list

    def test_regression_meta_attributes_order(self):
        # This bug was caused by hardcoded attributes order in re_meta pattern.
        # Example app that was affected was GitLab CI.
        content1 = "<meta content='GitLab Continuous Integration' name='description'>"
        content2 = "<meta name='description' content='GitLab Continuous Integration'>"

        results1 = self.detector.check_meta(content1)
        results2 = self.detector.check_meta(content2)

        expected = [{'app': 'GitLab CI', 'ver': None}]

        assert results1 == results2 == expected



from __future__ import absolute_import, division, print_function, unicode_literals
import six

import re
import os
from wad import tools
from wad.clues import _Clues
import itertools

CLUES_FILE = os.path.join(os.path.dirname(__file__), '../etc/apps.json')


def get_fields(app, key, rules_list):
    # returns a list of (app-ruletype-rule, fieldkey, fieldvalue) tuples,
    # e.g. ('LabVIEW-headers-Server', 'version', '\\1')
    return list(six.moves.map(
        lambda item: [("%s-%s-%s" % (app, key, item[0]), k1, v1) for (k1, v1) in six.iteritems(item[1]) if k1 != 're'],
        rules_list
    ))


def test_clues_correct():
    #
    # If there's an error, run py.test with -l option to see which field has failed.
    #
    clues = _Clues()
    clues.load_clues(CLUES_FILE)
    apps, categories = clues.apps, clues.categories
    types = {}
    fields = {
        'implies': {'expected_type': list, 'min_expected_amount': 220},
        'excludes': {'expected_type': list, 'min_expected_amount': 5},
        'script': {'expected_type': list, 'min_expected_amount': 205},
        'url': {'expected_type': list, 'min_expected_amount': 33},
        'headers': {'expected_type': dict, 'min_expected_amount': 255},
        'html': {'expected_type': list, 'min_expected_amount': 230},
        'meta': {'expected_type': dict, 'min_expected_amount': 160},
        'env': {'expected_type': list, 'min_expected_amount': 225},
    }
    basic_fields = {
        'website': {'min_expected_amount': len(apps)},
        'cats': {'min_expected_amount': len(apps)},
        'catsStr': {'min_expected_amount': len(apps)},
    }
    expected_str_types = set([six.text_type])

    for app in apps:
        for t in apps[app]:
            tools.count(types, t)

    # check if all categories listed in app are defined
    assert (set(six.moves.reduce(list.__add__, [apps[a]['cats'] for a in apps])) <=
            set([int(x) for x in categories.keys()]))

    # check if only expected fields are defined for apps
    assert set(six.iterkeys(types)) == set(itertools.chain(six.iterkeys(fields), six.iterkeys(basic_fields)))

    # check if numbers of entries are as expected
    assert len(apps) > 700
    for field, field_dict in itertools.chain(six.iteritems(fields), six.iteritems(basic_fields)):
        assert types[field] >= field_dict['min_expected_amount']

    # check if all implies are lists of unicodes, headers and meta are dictionaries of str/unicode,
    # and others (including app names) are str/unicode
    assert set([type(a) for a in apps]) <= expected_str_types

    for field, field_dict in six.iteritems(fields):
        assert set([type(apps[a][field]) for a in apps if field in apps[a]]) <= set([field_dict['expected_type']])
        assert set([type(v) for a in apps if field in apps[a] for v in apps[a][field]]) <= expected_str_types

    for field in ['headers', 'meta']:
        assert set(
            [type(apps[a][field][k]) for a in apps if field in apps[a] for k in apps[a][field]]) <= expected_str_types

    # check if all 'implies' and 'excludes' references exist
    for field in ['implies', 'excludes']:
        assert (set(six.moves.reduce(list.__add__, [apps[a][field] for a in apps if field in apps[a]])) <=
                set(apps.keys()))


def test_compile_clue():
    assert _Clues.compile_clue("abc") == {"re": re.compile("abc", flags=re.I)}
    assert _Clues.compile_clue("abc\;version:$1") == {"re": re.compile("abc", flags=re.I), "version": "$1"}
    assert _Clues.compile_clue("ab;c\;k:v1:v2\;aaa") == {"re": re.compile("ab;c", flags=re.I),
                                                         "k": "v1:v2", "aaa": None}


def test_compile_clues():
    clues = _Clues()
    clues.load_clues(CLUES_FILE)
    clues.compile_clues()
    apps = clues.apps

    # check optional fields like 'version' and 'confidence'
    fields = []
    for app in apps:
        for key in apps[app]:
            if key in ['script', 'html', 'url']:
                fields += six.moves.reduce(list.__add__, get_fields(app, key, list(enumerate(apps[app][key + '_re']))))
            if key in ['meta', 'headers']:
                fields += six.moves.reduce(list.__add__,
                                           get_fields(app, key,
                                                      [item for item in six.iteritems(apps[app][key + '_re'])]))

    assert len(fields) > 200

    # are there only 'version' and 'confidence' fields?
    assert set([k for (_, k, _) in fields]) == set(['confidence', 'version'])

    # are 'confidentiality' values always between 0 and 100?
    assert list(six.moves.filter(lambda x: not (0 < int(x) < 100),
                                 set([v for (_, k, v) in fields if k == 'confidence']))) == []

    # are 'version' values known/expected? or any new? if new, then test whether they work fine
    assert (set([v for (_, k, v) in fields if k == 'version']) ==
            set(['\\1 \\2', '\\1?Enterprise:Community', '\\1', '\\1?UA:', '\\1?4:5',
                 '\\1?4.1+:', '\\1.\\2.\\3', '\\1?2+:', 'API v\\1', '2+']))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Sebastian.Lopienski@cern.ch
#
from __future__ import absolute_import, division, print_function, unicode_literals

import logging
from optparse import OptionParser

import json

from wad import tools
from wad.clues import Clues, CLUES_FILE
from wad.detection import TIMEOUT, Detector


def main(timeout=TIMEOUT):
    desc = """WAD -
This component analyzes given URL(s) and detects technologies, libraries,
frameworks etc. used by this application, from the OS and web server level,
to the programming platform and frameworks, and server- and client-side
applications, tools and libraries. For example: OS=Linux, webserver=Apache,
platform=PHP, cms=Drupal, analytics=Google Analytics, javascript-lib=jQuery
etc."""

    parser = OptionParser(description=desc,
                          usage="Usage: %prog -u <URLs|@URLfile>\nHelp:  %prog -h",
                          version="%prog 1.0")

    parser.add_option("-u", "--url", dest="urls", metavar="URLS|@FILE",
                      help="list of URLs (comma-separated), or a file with a list of URLs (one per line)")

    parser.add_option("-l", "--limit", dest="limit", metavar="URLMASK",
                      help="in case of redirections, only include pages with URLs matching this mask - "
                           "e.g. 'https?://[^/]*\.abc\.com/'")

    parser.add_option("-x", "--exclude", dest="exclude", metavar="URLMASK",
                      help="in case of redirections, exclude pages with URL matching this mask - "
                           "e.g. 'https?://[^/]*/(login|logout)'")

    parser.add_option("-o", "--output", dest="output_file", metavar="FILE",
                      help="output file for detection results (default: STDOUT)")

    parser.add_option("-c", "--clues", dest="clues_file", metavar="FILE", default=CLUES_FILE,
                      help="clues for detecting web applications and technologies")

    parser.add_option("-t", "--timeout", action="store", dest="TIMEOUT", default=timeout,
                      help="set timeout (in seconds) for accessing a single URL")

    tools.add_log_options(parser)

    options = parser.parse_args()[0]

    tools.use_log_options(options)

    if not options.urls:
        parser.error("Argument -u missing")
        return

    timeout = int(options.TIMEOUT)

    if options.urls[0] == "@":
        try:
            f = open(options.urls[1:])
            urls = f.readlines()
            f.close()
        except Exception as e:
            # an I/O exception?
            logging.error("Error reading URL file %s, terminating: %s", options.urls[1:], tools.error_to_str(e))
            return
    else:
        urls = [x.strip() for x in options.urls.split(",") if x.strip() != ""]

    Clues.get_clues(options.clues_file)

    results = Detector().detect_multiple(urls, limit=options.limit, exclude=options.exclude, timeout=timeout)

    if options.output_file:
        try:
            f = open(options.output_file, "w")
            f.write(json.dumps(results))
            f.close()
            logging.debug("Results written to file %s", options.output_file)
        except Exception as e:
            # an I/O exception?
            logging.error("Error writing results to file %s, terminating: %s", options.output_file,
                          tools.error_to_str(e))
            return
    else:
        print(json.dumps(results, indent=4))


if __name__ == "__main__":
    main()



# Clues taken from Wappalyzer
#
# clues:        https://github.com/AliasIO/Wappalyzer/blob/master/src/apps.json
# more info:    https://github.com/AliasIO/Wappalyzer/blob/master/README.md
# detection:    https://github.com/AliasIO/Wappalyzer/blob/master/src/wappalyzer.js
# JavaScript RegExp object: http://www.w3schools.com/jsref/jsref_obj_regexp.asp
from __future__ import absolute_import, division, print_function, unicode_literals
import six

import os
import logging
import re
import json

from wad import tools

CLUES_FILE = os.path.join(os.path.dirname(__file__), 'etc/apps.json')


class _Clues(object):
    def __init__(self):
        self.apps = None
        self.categories = None

    def get_clues(self, filename=CLUES_FILE):
        if self.apps and self.categories:
            return self.apps, self.categories

        self.load_clues(filename)
        self.compile_clues()
        return self.apps, self.categories

    @staticmethod
    def read_clues_from_file(filename):
        logging.info("Reading clues file %s", filename)
        try:
            json_data = open(filename)
        except IOError as e:
            logging.error("Error while opening clues file, terminating: %s", tools.error_to_str(e))
            raise

        try:
            clues = json.load(json_data, encoding='utf-8')
        except ValueError as e:
            logging.error("Error while reading JSON file, terminating: %s", tools.error_to_str(e))
            raise

        json_data.close()
        categories = clues['categories']
        apps = clues['apps']
        return apps, categories

    @staticmethod
    def merge_dictionaries(dict1, dict2, desc):
        for key in dict2:
            if key in dict1:
                logging.warning("%s '%s' in both clues files", desc, key)
            dict1[key] = dict2[key]

    def string_to_array(self, tag):
        for app in self.apps:
            if tag in self.apps[app]:
                if type(self.apps[app][tag]) is six.text_type:
                    self.apps[app][tag] = [self.apps[app][tag]]

    def ignore_attributes(self, tag):
        # ignore all attributes (such as "confidence") specified after \;
        for app in self.apps:
            if tag in self.apps[app]:
                new_list = []
                for item in self.apps[app][tag]:
                    values = item.split("\;")
                    new_list += [values[0]]
                self.apps[app][tag] = new_list

    def add_categories_str(self):
        for app in self.apps:
            self.apps[app]['catsStr'] = six.moves.reduce(lambda a, b: "%s,%s" % (a, b),
                                                         [str(self.categories[str(x)]) for x in self.apps[app]['cats']])

    def load_clues(self, filename):
        self.apps, self.categories = self.read_clues_from_file(filename)

        additional_clues = filename + ".other"
        if os.path.isfile(additional_clues):
            apps2, categories2 = self.read_clues_from_file(additional_clues)
            # merge dictionaries with apps and categories; warn about repeated ones
            self.merge_dictionaries(self.apps, apps2, "App")
            self.merge_dictionaries(self.categories, categories2, "Category")

        # some clues are strings while others are array of strings - make them all arrays
        for field in ['url', 'html', 'env', 'script', 'implies', 'excludes']:
            self.string_to_array(field)

        # ignoring confidence in implies and excludes
        self.ignore_attributes('implies')
        self.ignore_attributes('excludes')

        # add categories string field
        self.add_categories_str()

    @staticmethod
    def compile_clue(regexp_extended):
        values = regexp_extended.split("\;")
        regex_dict = {"re": re.compile(values[0], flags=re.IGNORECASE)}
        for extra_field in values[1:]:
            try:
                (k, v) = extra_field.split(':', 1)
                regex_dict[k] = v
            except ValueError:
                regex_dict[extra_field] = None

        return regex_dict

    def compile_clues(self):
        # compiling regular expressions
        for app in self.apps:
            regexps = {}
            for key in self.apps[app]:
                if key in ['script', 'html', 'url']:
                    regexps[key + "_re"] = list(six.moves.map(self.compile_clue, self.apps[app][key]))
                if key in ['meta', 'headers']:
                    regexps[key + "_re"] = {}
                    for entry in self.apps[app][key]:
                        regexps[key + "_re"][entry] = self.compile_clue(self.apps[app][key][entry])
            self.apps[app].update(regexps)


Clues = _Clues()  # For use as singleton



from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', to='rst', format='md')
    long_description += "\n\n" + pypandoc.convert('AUTHORS.md', to='rst', format='md')
except (IOError, ImportError):
    long_description = ''

setup(
    name='wad',
    version='0.1.2',
    description='A tool for detecting technologies used by web applications.',
    long_description=long_description,
    url='https://github.com/CERN-CERT/WAD',
    license='GPLv3',
    author='Sebastian Lopienski',
    author_email='sebastian.lopienski@cern.ch',
    packages=find_packages(),
    include_package_data=True,
    requires=['six'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Security',
        'Topic :: Internet :: WWW/HTTP',
    ],
    entry_points={
        'console_scripts': [
            'wad = wad.__main__:main'
        ]
    },
)
